// IJU.java

package astroj;

import java.awt.*;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import javax.swing.JFrame;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.UIManager;
import java.util.Locale;

/**
 * Various static utilities
 */
public class IJU
	{
    public static Locale locale = Locale.US;
    public static DecimalFormatSymbols dfs = new DecimalFormatSymbols(locale);
    public static String separator = System.getProperty("file.separator");
    public static DecimalFormat uptoEightPlaces = new DecimalFormat("#####0.########", IJU.dfs);
    public static DecimalFormat twoPlaces = new DecimalFormat("######0.00", IJU.dfs);
    
       
	/**
	 * Extracts a double array from a string array.
	 */
	public static double[] extractDoubles (String[] s)
		{
		double[] arr = new double[s.length];
		try	{
			for (int i=0; i < arr.length; i++)
				arr[i] = Double.parseDouble(s[i]);
			}
		catch (NumberFormatException e)
			{
			arr = null;
			}        
		return arr;
		}      
    
   
	/**
	 * Extracts a boolean array from a string array.
	 */
	public static boolean[] extractBoolean (String[] s)
		{
		boolean[] arr = new boolean[s.length];

        for (int i=0; i < arr.length; i++)
            {
            if (s[i].equalsIgnoreCase("true"))
                arr[i] = true;
            else
                arr[i] = false;
            }
		return arr;
		}      
    
    
    public static double sexToDec(String text)
        {
        double X = Double.NaN;
        boolean XNegative = false;
        String[] pieces = text.replace("-", " -").replaceAll("[^0-9\\.\\-]{1,}", " ").trim().split("[^0-9\\.\\-]{1,}");
        if (pieces.length > 0)
            {
            X = parseDouble(pieces[0]);
            if (!Double.isNaN(X) &&  pieces[0].contains("-"))
                {
                X = -X;
                XNegative = true;
                }
            if (pieces.length > 1) X += Math.abs(parseDouble(pieces[1]))/60.0;
            if (pieces.length > 2) X += Math.abs(parseDouble(pieces[2]))/3600.0;
            }

        if (XNegative) X = -X;
        return X;
        }    
    
    public static String decToSexRA(double d)
        {
        return decToSex (d/15.0, 3, 24, false);
        }
    
public static String decToSexDec(double d)
        {
        return decToSex (d, 2, 90, true);
        }    
    
	public static String decToSex (double d, int fractionPlaces, int base, Boolean showPlus)
		{
        DecimalFormat nf = new DecimalFormat();
        DecimalFormat nf2x =  new DecimalFormat();
        nf.setDecimalFormatSymbols(IJU.dfs);
        nf2x.setDecimalFormatSymbols(IJU.dfs);
		nf.setMinimumIntegerDigits(2);
        nf2x.setMinimumIntegerDigits(2);
        nf2x.setMinimumFractionDigits(fractionPlaces);
        nf2x.setMaximumFractionDigits(fractionPlaces);

        boolean ampm = false;
        boolean pm = false;
        if (base == 1224)
            {
//            base = 12;
            ampm = true;
            if (d >= 12.0)
                {
                d -= 12.0;
                pm = true;
                }
            }

		double dd = Math.abs(d);
//        dd += 0.0000001;

		int h = (int)dd;
		int m = (int)(60.0*(dd-(double)h));
		double s = 3600.0*(dd-(double)h-(double)m/60.0);

        if (parseDouble(nf2x.format(s)) >= 60.0)
            {
            s = 0.0;
            m += 1;
            }
        if (m > 59)
            {
            m -= 60;
            h += 1;
            }
        if (d > 0 && h >= base)
            {
            if (base == 180 || (base == 12 && !ampm))
                {
                d = -d;
                if (s != 0)
                    {
                    s = 60 - s;
                    m = 59 - m;
                    h--;
                    }
                 else if (m != 0)
                    {
                    m = 59 - m;
                    h--;
                    }
                }
            else if (base == 12 && ampm)
                {
                h -= base;
                pm = !pm;
                }
            else if(base == 90)
                {
                h = 90;
                m = 0;
                s = 0;
                }
            else
                h -= base;
            }
        else if (base == 90 && d < -90)
            {
            h = 90;
            m = 0;
            s = 0;
            }

        if (ampm && h == 0) h = 12;
		String str = "";
		if (d < 0.0) str = "-";
        else if (showPlus) str = "+";
        str += "" + nf.format(h) + ":" + nf.format(m) + ":" + nf2x.format(s);
        if (ampm) str += pm ? " PM" : " AM";
		return str;
		}    
    

    
    
    
    
    public static double getTextSpinnerDoubleValue(JSpinner spinner)
        {
//        boolean debug = true;
//        DecimalFormat df = new DecimalFormat("0.0000", dfs);
//        DecimalFormatSymbols symbols = df.getDecimalFormatSymbols();
//        char decSep = symbols.getDecimalSeparator(); 
//        char thouSep = symbols.getGroupingSeparator();  
        String text = (String)spinner.getModel().getValue();
//        if (debug) IJ.log("Spinner Original Text = "+text);
//        text = text.replace(""+thouSep, "").replace(decSep, '.').trim();
//        if (debug) IJ.log("Spinner Modified Text = "+text);
        return parseDouble(text.trim()); 
        }   
    
    public static double getTextFieldDoubleValue(JTextField textField)
        {
//        boolean debug = true;
//        DecimalFormat df = new DecimalFormat("0.0000",dfs);
//        DecimalFormatSymbols symbols = df.getDecimalFormatSymbols();
//        char decSep = symbols.getDecimalSeparator(); 
//        char thouSep = symbols.getGroupingSeparator();  
        String text = textField.getText().trim();
//        if (debug) IJ.log("Spinner Original Text = "+text);
//        text = text.replace(""+thouSep, "").replace(decSep, '.').trim();
//        if (debug) IJ.log("Spinner Modified Text = "+text);
        return parseDouble(text); 
//        if (Double.isNaN(value)) 
//            {
////            if (debug) IJ.log("Value is NaN");
//            textField.setText("0"+decSep+"0");
//            value = 0.0;
//            }
//        if (debug) IJ.log("Value = "+value);
//        return value;
        }
    

    
    
    public static void setFrameSizeAndLocation(JFrame frame, int defaultX, int defaultY, int defaultWidth, int defaultHeight)
        {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice[] gds = ge.getScreenDevices();
        GraphicsDevice gd = null;
        Rectangle screenBounds = new Rectangle();
        boolean foundScreen = false;
        for (int j = 0; j < gds.length; j++) 
            {
            gd = gds[j];
            screenBounds.setRect(gd.getDefaultConfiguration().getBounds());
            if (screenBounds.contains(defaultX, defaultY)) 
                {
                foundScreen = true;
                break;
                }
            }
        if (!foundScreen)
            {
            gd = ge.getDefaultScreenDevice();
            screenBounds.setRect(gd.getDefaultConfiguration().getBounds());                    
            }

        Insets insets = Toolkit.getDefaultToolkit().getScreenInsets(gd.getDefaultConfiguration());  
        screenBounds.x += insets.left;  
        screenBounds.y += insets.top;  
        screenBounds.width -= insets.left + insets.right;  
        screenBounds.height -= insets.top + insets.bottom; 
        if (frame.getHeight() > screenBounds.height)
            {
            frame.setSize(frame.getWidth()+((Integer)UIManager.get("ScrollBar.width")).intValue(), screenBounds.height);
            }        
        if (frame.getWidth() > screenBounds.width)
            {
            frame.setSize(screenBounds.width, frame.getHeight());
            }        
        if (!screenBounds.contains(defaultX, defaultY))
            {
            defaultX = screenBounds.x + screenBounds.width/2 - frame.getWidth()/2;
            defaultY = screenBounds.y + screenBounds.height/2 - frame.getHeight()/2;
            }
        if (defaultWidth > 0 && defaultWidth < screenBounds.width)
            {
            frame.setSize(defaultWidth, frame.getHeight());
            }
        if (defaultHeight > 0 && defaultHeight < screenBounds.height)
            {
            frame.setSize(frame.getWidth(), defaultHeight);
            }        
        frame.setLocation(defaultX, defaultY);        
        }  
    
    



	/**
	 * Help routine to find minimum value in a double array.
	 */
	public static double minOf (double[] arr)
		{
		int n=arr.length;
		if (n == 0) return Double.NaN;

		double mn = arr[0];
		for (int i=1; i < n; i++)
			{
			if (!Double.isNaN(arr[i]) && arr[i] < mn) mn=arr[i];
			}
		return mn;
		}

	/**
	 * Help routine to find the maximum value in a double array.
	 */
	public static double maxOf (double[] arr)
		{
		int n=arr.length;
		if (n == 0) return Double.NaN;

		double mx = arr[0];
		for (int i=1; i < n; i++)
			{
			if (!Double.isNaN(arr[i]) && arr[i] > mx) mx=arr[i];
			}
		return mx;
		}
    
	public static int parseInteger(String s, int defaultValue) {
		if (s==null) return defaultValue;
		try {
			defaultValue = Integer.parseInt(s);
            } 
        catch (NumberFormatException e) {}
		return defaultValue;			
	}

	public static int parseInteger(String s) {
		return parseInteger(s, -1);
	}

    
	/**
	* Returns a double containg the value represented by the 
	* specified <code>String</code>.
	*
	* @param      s   the string to be parsed.
	* @param      defaultValue   the value returned if <code>s</code>
	*	does not contain a parsable double
	* @return     The double value represented by the string argument or
	*	<code>defaultValue</code> if the string does not contain a parsable double
	*/
	public static double parseDouble(String s, double defaultValue) {
		if (s==null) return defaultValue;
		try {
			defaultValue = Double.parseDouble(s);
		} catch (NumberFormatException e) {}
		return defaultValue;			
	}

	/**
	* Returns a double containg the value represented by the 
	* specified <code>String</code>.
	*
	* @param      s   the string to be parsed.
	* @return     The double value represented by the string argument or
	*	Double.NaN if the string does not contain a parsable double
	*/
	public static double parseDouble(String s) {
		return parseDouble(s, Double.NaN);
	}    
	}
